#!/bin/bash

# der Pfad des Hoerbuchs wird als Parameter erwartet
# z.B.: ./mp3tom4b /archiv/hoerbuchtitel
# fuer die Verarbeitung werden folgende Programme benoetigt:
# mp3info: Laenge einer MP3 Datei ermitteln
# madplay: MP3 als Wave Stream ausgeben
# mp3val: beschaedigte und zusammengesetzte MP3's fixen
# faac: Konvertieren des Wave Streams nach AAC bzw. M4B
# mp3wrap: Zusammenfgen mehrerer MP3 Dateien

# remove trailing /
audiobookpath=${1%/}
# Pfad auf Gueltigkeit pruefen
if [ ! -n "$audiobookpath" ]; then
 echo "Der Pfad zum Hoerbuchordner fehlt."
 exit 0
fi
if [ $audiobookpath = "." ]; then
 echo "Der Pfad muss absolut angegeben werden."
 exit 0
fi
if [[ $audiobookpath =~ [.][/] ]]; then
 echo "Der Pfad muss absolut angegeben werden."
 exit 0
fi
if [ ! -d $audiobookpath ]; then
 echo "Der eingegebene Ordner existiert nicht."
 exit 0
fi

# Nutzereingaben
while :   # Endlosschleife, bis weiter unten mit "j" bestaetigt wird
do
 tput clear   # Bildschirm loeschen
 echo "Die Dateien im Ordner $audiobookpath werden zu einem Audiobook im Format M4B konvertiert."
 echo -e "Autor des Buches: \c "
 read author
 echo -e "Buchtitel: \c "
 read booktitle
 echo -e "Jahr: \c "
 read year

 echo -e "Sind die Angaben korrekt (j/n/q)? \c "
 read answer
 case $answer in
  j*|J*|y*|Y*) break;;
  q*|Q*) exit 0;;
  *) continue;;
 esac
done

# Da iTunes offenbar nicht mit Hoerbuchdateien zurechtkommt, die laenger als 8 Stunden sind \
# werden die MP3 Dateien erst einmal auf mehrere Unterordner verteilt.
#
# Ein paar Variablenzuweisungen
# aendern des Separators auf Zeilenumbruch, sonst wird der Output von "find" falsch ausgegeben
IFS='
'
index=1
part=1
m4bfilename="$author - $booktitle"
mkdir -p $audiobookpath/tmp/teil1

# alle MP3 Dateien im uebergebenen Ordner suchen und sortieren
filelist=$(find "$audiobookpath" -name "*mp3" | sort)

# die Aufteilung auf n Unterordner
# neuen Unterordner erstellen, wenn die Laenge der Dateien ueber 8 Stunden liegt
for x in $filelist
do
 lengthofnewfile=`mp3info -p "%S" $x`
 let totallength=$totallength+$lengthofnewfile
 if [ $totallength -ge 28800 ]; then   # in Sekunden
  let part=$part+1
  totallength=`mp3info -p "%S" $x`
  mkdir -p $audiobookpath/tmp/teil${part}
 fi

 # Nullen voranstellen, damit die Sortierung erhalten bleibt
 if [ $index -lt 100 ]; then
    number=0$index
 fi
 if [ $index -lt 10 ]; then
    number=00$index
 fi
 if [ $index -ge 100 ]; then
    number=$index
 fi
 # die aktuelle Datei x aus der Liste in den jeweiligen Unterordner unter /tmp kopieren
 cp $x $audiobookpath/tmp/teil${part}/$number.mp3
 let index=$index+1
done

# Konvertierung
# Wenn gesamtes Hoerbuch weniger als 8 Stunden lang ist, aendert sich nur an den Parametern von faac etwas
# daher die if Verzweigung
if [ $part -gt 1 ]; then
 mkdir $audiobookpath/m4b
 for (( i = 1; i <= $part; i++ ))
 do
  path=$audiobookpath/tmp/teil${i}
  # Alle Dateien des jeweiligen Unterordners zu einer MP3 zusammenfassen
  mp3wrap $path/teil${i}.mp3 $path/*.mp3
  # Quelle fuer faac waehlen, evtl gab es jeweils nur eine inputdatei
  if [ -f $path/teil${i}_MP3WRAP.mp3 ]; then
   inputfile="$path/teil${i}_MP3WRAP.mp3"
  else
   inputfile=`ls $path/*.mp3`
  fi
  # inputfile fixen
  mp3val $inputfile -f
  # in eine .m4b Datei konvertieren
  madplay -m -o wave:- $inputfile | faac -b 48 -w \
  --artist "$author" --album "$booktitle" --title "Teil ${i}" \
  --track "${i}/$part"  --genre "Hoerbuch" --year "$year" \
  -o $audiobookpath/output.m4b -
  # output umbenennen
  mv $audiobookpath/output.m4b $audiobookpath/m4b/$m4bfilename\ \(Teil\ ${i}\).m4b
 done
else
 path="$audiobookpath/tmp/teil1"
 mp3wrap $path/teil1.mp3 $path/*.mp3
 # Quelle fuer faac waehlen, evtl gab es nur eine inputdatei
 if [ -f $path/teil1_MP3WRAP.mp3 ]; then
  inputfile="$path/teil1_MP3WRAP.mp3"
 else
  inputfile=`ls $path/*.mp3`
 fi
 # inputfile fixen
 mp3val $inputfile -f
 # in eine .m4b Datei konvertieren
 madplay -m -o wave:- $inputfile | faac -b 48 -w \
 --artist "$author" --album "$booktitle" --title "$booktitle" \
 --track "1/1"  --genre "Hoerbuch" --year "$year" \
 -o $audiobookpath/output.m4b -
 # output umbenennen
 mv $audiobookpath/output.m4b $audiobookpath/$m4bfilename.m4b
fi

# aufraeumen
rm -R $audiobookpath/tmp
echo "inputfile = $inputfile"
exit 0
